<?php

/**
 * @author Pressmind Gmbh
 *
 * @example:
 * $id = 436010; // pressmind MediaObjectID
 * RestClient::$apiUrl = "http://xxx";
 * RestClient::$apiKey = "xxx";
 * RestClient::$apiVersion = "1.1.004";
 * RestClient::$username = "johndoe@presssmind.de";
 * RestClient::$password = "xx";
 * $r = RestClient::get("Text", array("id" => $id));
 * print_r($r);
 *
 */

class RestClient{

    /**
     * API Endpoint
     * @var string
     */
    public static $apiUrl;

    /**
     * pressmind API version
     * apiVersion is the same as the pressmind client version
     * @var string Example "1.1.004"
     */
    public static $apiVersion;

    /**
     * API Key
     * @var string
     */
    public static $apiKey;

    /**
     * Username
     * @var string
     */
    public static $username;


    /**
     * Password
     * @var string
     */
    public static $password;


    private static $cURL;

    const METHODE_GET    = 'GET';
    const METHODE_PUT    = 'PUT';
    const METHODE_POST   = 'POST';
    const METHODE_DELETE = 'DELETE';

    protected static $validMethods = array(
            self::METHODE_GET,
            self::METHODE_PUT,
            self::METHODE_POST,
            self::METHODE_DELETE
    );


    /**
     * @param string $apiController e.g. "Text"
     * @param string $httpMethod e.g. "GET"
     * @param array $data POST DATA
     * @param array $params GET Parameter
     * @throws Exception
     * @see Shorthand CRUD Functions self::get, post, etc
         * @return Response
    */

    public static function call($apiController, $httpMethod = self::METHODE_GET, $data, $params = array()) {

        // Init
        self::$apiUrl = rtrim(self::$apiUrl, '/').'/';

        //Initializes the cURL instance
        self::$cURL = curl_init();
        curl_setopt(self::$cURL, CURLOPT_FOLLOWLOCATION, false);
        curl_setopt(self::$cURL, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST);
        curl_setopt(self::$cURL, CURLOPT_ENCODING, 'gzip');
        curl_setopt(self::$cURL, CURLOPT_USERPWD, self::$username . ':' . self::$password);

        if (!in_array($httpMethod, self::$validMethods)) {
            throw new Exception('Invalid HTTP-Method: ' . $httpMethod);
        }

        $queryString = '';
        if (!empty($params)) {
            foreach($params as $key => $param){

                if(is_array($param) === true){
                    $param = implode(",", $param);
                }

                if($param == ""){
                    continue;
                }

                $queryString .= urlencode($key)."/".urlencode($param)."/";
            }
        }
        if(empty($queryString) === false){
            $queryString= rtrim($queryString, '/') . '/';
        }
        $url = self::$apiUrl.self::$apiVersion.'/rest/'.$apiController.'/api/'.self::$apiKey.'/'.$queryString;

        $dataString = self::prepareRequestBody($data);

        curl_setopt(self::$cURL, CURLOPT_URL, $url);
        curl_setopt(self::$cURL, CURLOPT_CUSTOMREQUEST, $httpMethod);
        curl_setopt(self::$cURL, CURLOPT_POSTFIELDS, $dataString);
        curl_setopt(self::$cURL, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt(self::$cURL, CURLOPT_SSL_VERIFYPEER , false);

        curl_setopt(self::$cURL, CURLOPT_HTTPHEADER, array(
        'Content-Type: application/json; charset=utf-8'
                ));

        $result   = curl_exec(self::$cURL);
        $httpCode = curl_getinfo(self::$cURL, CURLINFO_HTTP_CODE);
		
		$encode		=	@gzinflate(substr($result, 10, -8));

		if($encode !== false)
		{
			$result	=	$encode;
		}

        if(curl_errno(self::$cURL)){
            throw new Exception('Curl error: ' . curl_error(self::$cURL));
        }
        return self::prepareResponse($result, $httpCode, $url, $httpMethod);
    }



    /**
     * READ
     * @param string $apiController e.g. "Text"
     * @param array $params GET Parameter
     * @throws Exception
     * @return Response
     */

    public static function get($apiController, $params = array()) {
        return self::call($apiController, self::METHODE_GET, array(), $params);
    }

    /**
     * Create
     * @param string $apiController e.g. "Text"
     * @param mixed $data POST Data
     * @param array $params GET Parameter
     * @throws Exception
     * @return Response
     */
    public static function post($apiController, $data, $params = array()) {
        return self::call($apiController, self::METHODE_POST, $data, $params);
    }


    /**
     * Update
     * @param string $apiController e.g. "Text"
     * @param mixed $data PUT Data
     * @param array $params GET Parameter
     * @throws Exception
     * @return Response
     */
    public static function put($apiController, $data, $params = array()) {
        return self::call($apiController, self::METHODE_PUT, $data, $params);
    }


    /**
     * Delete
     * @param string $apiController e.g. "Text"
     * @param array $params GET Parameter
     * @throws Exception
     * @return object
     */
    public static function delete($apiController, $params = array()) {
        return self::call($apiController, self::METHODE_DELETE, array(), $params);
    }

    /**
     * @param string $result
     * @param string $httpCode
     * @return Response
     */
    protected static function prepareResponse($result, $httpCode, $url, $httpMethod) {

        $output = new Response();
        $output->url = $url;
        $output->http = $httpCode;
        $output->method = $httpMethod;
        $output->raw = $result;

        if(null === $decodedResult = json_decode($result, true)) {
            $jsonErrors = array(
                    JSON_ERROR_NONE => 'No error has occurred',
                    JSON_ERROR_DEPTH => 'The maximum stack depth has been exceeded',
                    JSON_ERROR_CTRL_CHAR => 'Control character error, possibly incorrectly encoded',
                    JSON_ERROR_SYNTAX => 'Syntax error',
                    JSON_ERROR_STATE_MISMATCH => 'Invalid or malformed JSON',
                    JSON_ERROR_UTF8 => 'Malformed UTF-8 characters, possibly incorrectly encoded'
            );
            $output->msg = "json_last_error: " . $jsonErrors[json_last_error()];

            return $output;
        }

        $output->error = false;
        $output->decodedResult = $decodedResult;

        return $output;
    }

    /**
     *
     * Preparing request as json
     * @param mixed $data
     * @return string json
     */
    protected static function prepareRequestBody($data){
        $json = json_encode($data, JSON_HEX_TAG|JSON_HEX_APOS|JSON_HEX_QUOT|JSON_HEX_AMP);
        return $json;
    }
    
   
}

/**
 * Value Object Response Object
 */
class Response{
	public $error = true;
	public $http = null;
	public $msg = null;
	public $raw = null;
	public $url = null;
	public $method = null;
	public $decodedResult = null;
}

?>